import json
from itertools import islice
from pprint import pprint

import openpyxl


class ExcelConverter(object):
    def __init__(self, file_name):
        self.file_name = file_name
        self.excel_json = {
            "projectproperties": {
                "name": "Touchless Project",
                "version": "1.0.0"
            },
            "pages": []
        }
        self.workbook = openpyxl.load_workbook(filename=self.file_name)

    def meta_row_obj(self, repeats_info):
        meta_info = {
            "title": next(filter(lambda x: x['header'] == 'Label', repeats_info)).get('val'),
            "description": "",
            "properties": {
                "join": str(next(filter(lambda x: x['header'] == 'Join', repeats_info)).get('val'))
            }
        }
        return meta_info

    def row_to_obj(self, row_data):
        col_obj  = {
            "cols": []
        }
        for col_groups in row_data:
            col_json = {
                "type": None,
                "properties": {
                }
            }
            for col in col_groups:
                header_val = col['header']
                col_val = col['val']

                if header_val == "Control Type":
                    col_json['type'] = col_val.lower()
                elif header_val == 'Join':
                    col_json['properties']['join'] = str(col_val)
                elif header_val == 'Label':
                    col_json['properties']['label'] = col_val
                else:
                    y_n_translation = {"Y": "yes", "N": "no"}.get(col_val.upper(), None)
                    col_val = y_n_translation if y_n_translation is not None else col_val
                    col_json['properties'][header_val.lower().replace(' ', '-')] = col_val

            col_obj["cols"].append(col_json)
        
        return col_obj

    def convert(self):
        sheets = self.workbook.sheetnames
        for sheet_name in sheets:
            excel_sheet = self.workbook[sheet_name]
            sheet_json = {
                "rows": []
            }
            row_json = {
            }
            for row in islice(excel_sheet.iter_rows(), 1, None):
                temp_row = [{'cell': c, 'header': excel_sheet.cell(1, c.column).value, 'val':c.value} for c in row]
                temp_row = list(filter(lambda x: x['val'] is not None, temp_row))
                slices = [idx for idx, vals in enumerate(temp_row) if vals['header'] == temp_row[0]['header']] + [len(temp_row)]
                repeats = [temp_row[slices[i]:slices[(i + 1) % len(slices)]] for i in range(len(slices) - 1)]

                # pprint(repeats)
                if len(repeats) == 1 and next(map(lambda x: x['val'] == "Header", repeats[0])):
                    update_val = self.meta_row_obj(repeats[0])
                    sheet_json.update(update_val)
                else:
                    if repeats:
                        gen_controls = self.row_to_obj(repeats)
                        sheet_json["rows"].append(gen_controls)
                # for row_repeats in repeats:
            self.excel_json["pages"].append(sheet_json)



import json
import os
import sys

from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtPrintSupport import *
from PyQt5.QtWidgets import *

from excel2json_gui.excel_converter import ExcelConverter


class ExcelConverterGui(QMainWindow):
    def __init__(self, *args, **kwargs):
        super(ExcelConverterGui, self).__init__(*args, **kwargs)
        self.script_dir = os.path.dirname(__file__)
        self.width = 450
        self.height = 300

        # self.setWindowIcon(QIcon(os.path.join('excel2json_gui/images', 'icon.png')))
        # self.setWindowIconText("Excel 2 JSON")
        # self.setWindowTitle("Excel 2 JSON")
        self.setWindowTitle(QCoreApplication.applicationName())
        self.converter = None
        self.excel_loaded = False
        
        self.layout = QVBoxLayout()
        self.excel_pages = {}

        self.tabs = QTabWidget()
        self.page_tabs = QTabWidget()
        self.excel_tab = QWidget()
        self.json_tab = QWidget()
        self.tabs.resize(300, 200)


        self.tabs.addTab(self.excel_tab, "Excel Viewer")
        self.tabs.addTab(self.json_tab, "JSON Viewer")

        self.create_table()
        self.excel_tab.layout = QVBoxLayout()
        self.excel_tab.layout.addWidget(self.page_tabs)

        self.excel_tab.setLayout(self.excel_tab.layout)

        self.json_tab.layout = QVBoxLayout()

        self.create_tree_view()
        self.json_tab.layout.addWidget(self.tree_widget)
        self.json_tab.setLayout(self.json_tab.layout)

        self.layout.addWidget(self.tabs)

        self.path = None

        container = QWidget()
        container.setLayout(self.layout)
        self.setCentralWidget(container)

        self.status = QStatusBar()
        self.setStatusBar(self.status)

        menu_bar = self.menuBar()
        # menu_bar.text("Excel 2 JSON")
        file_toolbar = QToolBar("File")
        file_toolbar.setIconSize(QSize(28, 28))
        self.addToolBar(file_toolbar)
        file_menu = menu_bar.addMenu("&File")

        open_file_action = QAction(
            # QIcon(self.script_dir.split('/')[-1] + '/images/folder-open-big.png'), "Open file...", self)
            QIcon(os.path.join('excel2json_gui/images', 'folder-open-big.png')), "Open file...", self)
            # QIcon(os.path.join(self.script_dir, '../images/folder-open-big.png')), "Open file...", self)
        open_file_action.setStatusTip("Open file")
        open_file_action.triggered.connect(self.file_open)
        file_menu.addAction(open_file_action)
        file_toolbar.addAction(open_file_action)

        convert_action = QAction(QIcon(os.path.join('excel2json_gui/images', 'convert-big.png')), "Convert Excel to JSON", self)
        # convert_action = QAction(QIcon(self.script_dir.split('/')[-1] + '/images/convert-big.png'), "Convert Excel to JSON", self)
        # convert_action = QAction(QIcon(os.path.join(self.script_dir, '../images/convert-big.png')), "Convert Excel to JSON", self)
        convert_action.setStatusTip("Convert the Excel file")
        convert_action.triggered.connect(self.convert_file)
        file_menu.addAction(convert_action)
        file_toolbar.addAction(convert_action)

        save_file_action = QAction(QIcon(os.path.join('excel2json_gui/images', 'save-big.png')), "Save", self)
        # save_file_action = QAction(QIcon(self.script_dir.split('/')[-1] + '/images/save-big.png'), "Save", self)
        # save_file_action = QAction(QIcon(os.path.join(self.script_dir, '../images/save-big.png')), "Save", self)
        save_file_action.setStatusTip("Save JSON")
        save_file_action.triggered.connect(self.file_save)
        file_menu.addAction(save_file_action)
        file_toolbar.addAction(save_file_action)
        print(self.script_dir)
        self.show()

    def get_image_path(self, var):
        relative_path = f"/images/{type(var).__name__}.png"
        # return os.path.join(self.script_dir, relative_path)
        return self.script_dir.split('/')[-1] + relative_path
        # return relative_path

    def dialog_critical(self, s):
        print("critical dialog")
        dlg = QMessageBox(self)
        dlg.setText(s)
        dlg.setIcon(QMessageBox.Critical)
        dlg.show()

    def create_table(self):
        if self.converter is not None:
            self.page_tabs.removeTab(0)
            for sheet in self.converter.workbook.worksheets:
                sheet_tab = QWidget()
                self.page_tabs.addTab(sheet_tab, sheet.title)
                sheet_tab.layout = QVBoxLayout()
                new_table = QTableWidget(sheet.max_row, sheet.max_column)
                for row in sheet.rows:
                    for cell in row:
                        row_val, col_val = cell.row - 1, cell.column - 1
                        cell_val = str(cell.value) if cell.value is not None else cell.value
                        new_table.setItem(row_val, col_val, QTableWidgetItem(cell_val))
                sheet_tab.layout.addWidget(new_table)
                sheet_tab.setLayout(sheet_tab.layout)
        else:
            blank_tab = QWidget()
            self.page_tabs.addTab(blank_tab, 'Empty')
            blank_tab.layout = QVBoxLayout()
            empty_table = QTableWidget(3, 5)
            for r in range(1, 4):
                for c in range(1, 5):
                    empty_table.setItem(r, c, QTableWidgetItem(None))
            blank_tab.layout.addWidget(empty_table)
            blank_tab.setLayout(blank_tab.layout)


    def create_tree_view(self):
        self.tree_widget = QTreeWidget()
        self.tree_widget.setHeaderLabels(["JSON Viewer"])

    def fill_tree(self, item, value):
        item.setExpanded(False)
        if isinstance(value, dict):
            for key, val in value.items():
                child = QTreeWidgetItem()
                icon = QIcon(os.path.join('excel2json_gui/images', f"{type(val).__name__}.png"))
                # icon = QIcon(self.get_image_path(val))
                child.setIcon(0, icon)
                if any(map(lambda x: isinstance(val, x), (str, bool, int))):
                    child.setText(0, f'{key}: "{val}"' if isinstance(val, str) else f'{key}: {val}')
                    item.addChild(child)
                else:
                    child.setText(0, f"{key}")
                    item.addChild(child)
                    self.fill_tree(child, val)
        elif isinstance(value, list):
            for idx, val in enumerate(value):
                child = QTreeWidgetItem()
                item.addChild(child)
                icon = QIcon(os.path.join('excel2json_gui/images', f"{type(val).__name__}.png"))
                # icon = QIcon(self.get_image_path(val))
                child.setIcon(0, icon)
                if isinstance(val, dict):
                    child.setText(0, f'{idx}')
                    self.fill_tree(child, val)
                elif isinstance(val, list):
                    child.setText(0, '[list]')
                    self.fill_tree(child, val)
                else:
                    child.setText(0, f"{val}")
        else:
            child = QTreeWidgetItem()
            child.setText(0, f"{value}")
            icon = QIcon(os.path.join('excel2json_gui/images', f"{type(value).__name__}.png"))
            # icon = QIcon(self.get_image_path(value))
            child.setIcon(0, icon)
            item.addChild(child)


    def file_open(self):
        path, _ = QFileDialog.getOpenFileName(
            self, "Open file", "", "Text documents (*.txt);All files (*.*)")

        if path:
            try:
                self.converter = ExcelConverter(file_name=path)
                self.excel_loaded = True
                self.create_table()
            except Exception as e:
                self.dialog_critical(str(e))

    def file_save(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        save_dialog, _ = QFileDialog.getSaveFileName(
            self,
            "QFileDialog.getSaveFileName()",
            "",
            "JSON (*.json)",
            options=options
        )
        with open(save_dialog, 'w') as f:
            json.dump(self.converter.excel_json, f, indent=4)

    def convert_file(self):
        if self.converter is not None:
            self.converter.convert()
            self.out_json = json.dumps(self.converter.excel_json, indent=4)
            self.fill_tree(self.tree_widget.invisibleRootItem(), self.converter.excel_json)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['excel2json_gui']

package_data = \
{'': ['*'], 'excel2json_gui': ['images/*']}

install_requires = \
['PyQt5>=5.15.0,<6.0.0', 'openpyxl>=3.0.4,<4.0.0']

entry_points = \
{'console_scripts': ['excel2json-gui = excel2json_gui.main:main']}

setup_kwargs = {
    'name': 'excel2json-gui',
    'version': '0.1.6',
    'description': 'GUI program to convert CH5 spreadsheet to JSON',
    'long_description': None,
    'author': 'mflorczak',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from PyQt5.QtGui import *
from PyQt5.QtPrintSupport import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *

from excel2json_gui.gui import ExcelConverterGui
import sys

def main():
    app = QApplication(sys.argv)
    app.setApplicationName("Excel 2 JSON")

    
    window = ExcelConverterGui()
    # app.exec_()
    # window = ExcelConverterGui()
    # window.show()
    sys.exit(app.exec_())